@@ -3,6 +3,50 @@ import urllib
 
 from GithubObject import *
 
+Event = GithubObject(
+    "Event",
+    InternalSimpleAttributes(
+        "type", "public", "payload", "created_at", "id", "commit_id", "url",
+        "event", "issue",
+    ),
+)
+
+def __testHook( hook ):
+    hook._github._statusRequest( "POST", hook._baseUrl() + "/test", None, None )
+Hook = GithubObject(
+    "Hook",
+    BaseUrl( lambda obj: obj._repo._baseUrl() + "/hooks/" + str( obj.id ) ),
+    InternalSimpleAttributes(
+        "url", "updated_at", "created_at", "name", "events", "active", "config",
+        "id", "last_response",
+        "_repo", ### Ugly hack
+    ),
+    Editable( [ "name", "config" ], [ "events", "add_events", "remove_events", "active" ] ),
+    Deletable(),
+    SeveralAttributePolicies( [ MethodFromCallable( "test", [], [], __testHook, SimpleTypePolicy( None ) ) ], "Testing" )
+)
+
+Authorization = GithubObject(
+    "Authorization",
+    BaseUrl( lambda obj: "/authorizations/" + str( obj.id ) ), ### @todo make the lambda return a tuple, and BaseUrl convert elements to strings and join them with "/"
+    InternalSimpleAttributes(
+        "id", "url", "scopes", "token", "app", "note", "note_url", "updated_at",
+        "created_at",
+    ),
+    Editable( [], [ "scopes", "add_scopes", "remove_scopes", "note", "note_url" ] ),
+    Deletable(),
+)
+
+UserKey = GithubObject(
+    "UserKey",
+    BaseUrl( lambda obj: "/user/keys/" + str( obj.id ) ),
+    InternalSimpleAttributes(
+        "url", "id", "title", "key",
+    ),
+    Editable( [], [ "title", "key" ] ),
+    Deletable(),
+)
+
 AuthenticatedUser = GithubObject(
     "AuthenticatedUser",
     BaseUrl( lambda obj: "/user" ),
@@ -20,6 +64,21 @@ AuthenticatedUser = GithubObject(
         SeveralElementsAddable(),
         SeveralElementsRemovable()
     ),
+    ExternalListOfObjects( "authorizations", "authorization", Authorization,
+        ListGetable( [], [] ),
+        ElementGetable( [ "id" ], [] ),
+        ElementCreatable( [], [ "scopes", "note", "note_url" ] ),
+        url = "/authorizations",
+    ),
+    ExternalListOfObjects( "keys", "key", UserKey,
+        ListGetable( [], [] ),
+        ElementGetable( [ "id" ], [] ),
+        ElementCreatable( [ "title", "key" ], [] ),
+    ),
+    ExternalListOfObjects( "events", "event", Event,
+        ListGetable( [], [] ),
+        url = "/events"
+    ),
 )
 
 NamedUser = GithubObject(
@@ -63,6 +122,34 @@ NamedUser._addAttributePolicy(
         ListGetable( [], [] )
     )
 )
+NamedUser._addAttributePolicy(
+    ExternalListOfObjects( "events", "event", Event,
+        ListGetable( [], [] )
+    ),
+)
+def __getPublicEvents( user ):
+    return [
+        Event( user._github, attributes, lazy = True )
+        for attributes
+        in user._github._dataRequest( "GET", user._baseUrl() + "/events/public", None, None )
+    ]
+NamedUser._addAttributePolicy(
+    MethodFromCallable( "get_public_events", [], [], __getPublicEvents, SimpleTypePolicy( "list of `Event`" ) )
+)
+NamedUser._addAttributePolicy(
+    ExternalListOfObjects( "received_events", "received_event", Event,
+        ListGetable( [], [] )
+    )
+)
+def __getPublicReceivedEvents( user ):
+    return [
+        Event( user._github, attributes, lazy = True )
+        for attributes
+        in user._github._dataRequest( "GET", user._baseUrl() + "/received_events/public", None, None )
+    ]
+NamedUser._addAttributePolicy(
+    MethodFromCallable( "get_public_received_events", [], [], __getPublicReceivedEvents, SimpleTypePolicy( "list of `Event`" ) )
+)
 
 Organization = GithubObject(
     "Organization",
@@ -71,7 +158,7 @@ Organization = GithubObject(
     InternalSimpleAttributes(
         "login", "id", "url", "avatar_url", "name", "company", "blog",
         "location", "email", "public_repos", "public_gists", "followers",
-        "following", "html_url", "created_at", "type",
+        "following", "html_url", "created_at", "type", "gravatar_id",
         # Seen only by owners
         "disk_usage", "collaborators", "billing_email", "plan", "private_gists",
         "total_private_repos", "owned_private_repos",
@@ -88,6 +175,9 @@ Organization = GithubObject(
         ElementRemovable(),
         ElementHasable()
     ),
+    ExternalListOfObjects( "events", "event", Event,
+        ListGetable( [], [] )
+    ),
 )
 
 AuthenticatedUser._addAttributePolicy(
@@ -101,9 +191,19 @@ NamedUser._addAttributePolicy(
     )
 )
 
+def __getOrganizationEvents( user, org ):
+    return [
+        Event( user._github, attributes, lazy = True )
+        for attributes
+        in user._github._dataRequest( "GET", "/users/" + user.login + "/events/orgs/" + org.login, None, None )
+    ]
+AuthenticatedUser._addAttributePolicy(
+    MethodFromCallable( "get_organization_events", [ "org" ], [], __getOrganizationEvents, SimpleTypePolicy( "list of `Event`" ) )
+)
+
 GitRef = GithubObject(
     "GitRef",
-    BaseUrl( lambda obj: obj._repo._baseUrl + "/git/" + obj.ref ),
+    BaseUrl( lambda obj: obj._repo._baseUrl() + "/git/" + obj.ref ),
     InternalSimpleAttributes(
         "ref", "url",
         "object", ### @todo Structure
@@ -112,22 +212,9 @@ GitRef = GithubObject(
     Editable( [ "sha" ], [ "force" ] ),
 )
 
-GitCommit = GithubObject(
-    "GitCommit",
-    BaseUrl( lambda obj: obj._repo._baseUrl + "/git/commits/" + obj.sha ),
-    InternalSimpleAttributes(
-        "sha", "url", "message",
-        "author", ### @todo Structure
-        "committer", ### @todo Structure
-        "tree", ### @todo Structure
-        "parents", ### @todo Structure
-        "_repo", ### Ugly hack
-    ),
-)
-
 GitTree = GithubObject(
     "GitTree",
-    BaseUrl( lambda obj: obj._repo._baseUrl + "/git/trees/" + obj.sha ),
+    BaseUrl( lambda obj: obj._repo._baseUrl() + "/git/trees/" + obj.sha ),
     InternalSimpleAttributes(
         "sha", "url",
         "tree", ### @todo Structure
@@ -135,9 +222,21 @@ GitTree = GithubObject(
     ),
 )
 
+GitCommit = GithubObject(
+    "GitCommit",
+    BaseUrl( lambda obj: obj._repo._baseUrl() + "/git/commits/" + obj.sha ),
+    InternalSimpleAttributes(
+        "sha", "url", "message",
+        "parents", ### @todo Structure
+        "author", "committer",
+        "_repo", ### Ugly hack
+    ),
+    InternalObjectAttribute( "tree", GitTree ),
+)
+
 GitBlob = GithubObject(
     "GitBlob",
-    BaseUrl( lambda obj: obj._repo._baseUrl + "/git/blobs/" + obj.sha ),
+    BaseUrl( lambda obj: obj._repo._baseUrl() + "/git/blobs/" + obj.sha ),
     InternalSimpleAttributes(
         "sha", "size", "url",
         "content", "encoding",
@@ -147,7 +246,7 @@ GitBlob = GithubObject(
 
 GitTag = GithubObject(
     "GitTag",
-    BaseUrl( lambda obj: obj._repo._baseUrl + "/git/tags/" + obj.sha ),
+    BaseUrl( lambda obj: obj._repo._baseUrl() + "/git/tags/" + obj.sha ),
     InternalSimpleAttributes(
         "tag", "sha", "url",
         "message",
@@ -159,7 +258,7 @@ GitTag = GithubObject(
 
 Label = GithubObject(
     "Label",
-    BaseUrl( lambda obj: obj._repo._baseUrl + "/labels/" + obj._identity ),
+    BaseUrl( lambda obj: obj._repo._baseUrl() + "/labels/" + obj._identity ),
     Identity( lambda obj: urllib.quote( obj.name ) ),
     InternalSimpleAttributes(
         "url", "name", "color",
@@ -173,7 +272,7 @@ __modifyAttributesForObjectsReferingReferedRepo = { "_repo": lambda obj: obj._re
 
 Milestone = GithubObject(
     "Milestone",
-    BaseUrl( lambda obj: obj._repo._baseUrl + "/milestones/" + str( obj.number ) ),
+    BaseUrl( lambda obj: obj._repo._baseUrl() + "/milestones/" + str( obj.number ) ),
     InternalSimpleAttributes(
         "url", "number", "state", "title", "description", "open_issues",
         "closed_issues", "created_at", "due_on",
@@ -189,7 +288,7 @@ Milestone = GithubObject(
 
 IssueComment = GithubObject(
     "IssueComment",
-    BaseUrl( lambda obj: obj._repo._baseUrl + "/issues/comment" + str( obj.id ) ),
+    BaseUrl( lambda obj: obj._repo._baseUrl() + "/issues/comments/" + str( obj.id ) ),
     InternalSimpleAttributes(
         "url", "body", "created_at", "updated_at", "id",
         "_repo", ### Ugly hack
@@ -199,9 +298,19 @@ IssueComment = GithubObject(
     Deletable(),
 )
 
+IssueEvent = GithubObject(
+    "IssueEvent",
+    BaseUrl( lambda obj: obj._repo._baseUrl() + "/issues/events/" + str( obj.id ) ),
+    InternalSimpleAttributes(
+        "id", "url", "created_at", "issue", "event", "commit_id",
+        "_repo", # Ugly hack
+    ),
+    InternalObjectAttribute( "actor", NamedUser ),
+)
+
 Issue = GithubObject(
     "Issue",
-    BaseUrl( lambda obj: obj._repo._baseUrl + "/issues/" + str( obj.number ) ),
+    BaseUrl( lambda obj: obj._repo._baseUrl() + "/issues/" + str( obj.number ) ),
     InternalSimpleAttributes(
         "url", "html_url", "number", "state", "title", "body", "labels",
         "comments", "closed_at", "created_at", "updated_at", "id", "closed_by",
@@ -224,11 +333,14 @@ Issue = GithubObject(
         ElementGetable( [ "id" ], [], __modifyAttributesForObjectsReferingReferedRepo ),
         ElementCreatable( [ "body" ], [], __modifyAttributesForObjectsReferingReferedRepo ),
     ),
+    ExternalListOfObjects( "events", "event", IssueEvent,
+        ListGetable( [], [], __modifyAttributesForObjectsReferingReferedRepo )
+    ),
 )
 
 Download = GithubObject(
     "Download",
-    BaseUrl( lambda obj: obj._repo._baseUrl + "/downloads/" + str( obj.id ) ),
+    BaseUrl( lambda obj: obj._repo._baseUrl() + "/downloads/" + str( obj.id ) ),
     InternalSimpleAttributes(
         "url", "html_url", "id", "name", "description", "size",
         "download_count", "content_type", "policy", "signature", "bucket",
@@ -241,7 +353,7 @@ Download = GithubObject(
 
 CommitComment = GithubObject(
     "CommitComment",
-    BaseUrl( lambda obj: obj._repo._baseUrl + "/comments/" + str( obj.id ) ),
+    BaseUrl( lambda obj: obj._repo._baseUrl() + "/comments/" + str( obj.id ) ),
     InternalSimpleAttributes(
         "url", "id", "body", "path", "position", "commit_id",
         "created_at", "updated_at", "html_url", "line",
@@ -254,7 +366,7 @@ CommitComment = GithubObject(
 
 Commit = GithubObject(
     "Commit",
-    BaseUrl( lambda obj: obj._repo._baseUrl + "/commits/" + str( obj.sha ) ),
+    BaseUrl( lambda obj: obj._repo._baseUrl() + "/commits/" + str( obj.sha ) ),
     InternalSimpleAttributes(
         "sha", "url",
         "parents", ### @todo Structure
@@ -266,8 +378,8 @@ Commit = GithubObject(
     InternalObjectAttribute( "author", NamedUser ),
     InternalObjectAttribute( "committer", NamedUser ),
     ExternalListOfObjects( "comments", "comment", CommitComment,
-        ListGetable( [], [] ),
-        ElementCreatable( [ "body", "commit_id", "line", "path", "position" ], [] ),
+        ListGetable( [], [], __modifyAttributesForObjectsReferingReferedRepo ),
+        ElementCreatable( [ "body" ], [ "commit_id", "line", "path", "position" ], __modifyAttributesForObjectsReferingReferedRepo ),
     ),
 )
 
@@ -300,7 +412,7 @@ PullRequestFile = GithubObject(
 
 PullRequestComment = GithubObject(
     "PullRequestComment",
-    BaseUrl( lambda obj: obj._repo._baseUrl + "/pulls/comments/" + str( obj.id ) ),
+    BaseUrl( lambda obj: obj._repo._baseUrl() + "/pulls/comments/" + str( obj.id ) ),
     InternalSimpleAttributes(
         "url", "id", "body", "path", "position", "commit_id",
         "created_at", "updated_at", "html_url", "line",
@@ -311,9 +423,13 @@ PullRequestComment = GithubObject(
     Deletable(),
 )
 
+def __pullRequestIsMerged( r ):
+    return r._github._statusRequest( "GET", r._baseUrl() + "/merge", None, None ) == 204
+def __mergePullRequest( r, **data ):
+    r._github._statusRequest( "PUT", r._baseUrl() + "/merge", None, data )
 PullRequest = GithubObject(
     "PullRequest",
-    BaseUrl( lambda obj: obj._repo._baseUrl + "/pulls/" + str( obj.number ) ),
+    BaseUrl( lambda obj: obj._repo._baseUrl() + "/pulls/" + str( obj.number ) ),
     InternalSimpleAttributes(
         "id", "url", "html_url", "diff_url", "patch_url", "issue_url", "number",
         "state", "title", "body", "created_at", "updated_at", "closed_at",
@@ -335,6 +451,19 @@ PullRequest = GithubObject(
         ElementGetable( [ "id" ], [], __modifyAttributesForObjectsReferingReferedRepo ),
         ElementCreatable( [ "body", "commit_id", "path", "position" ], [], __modifyAttributesForObjectsReferingReferedRepo ),
     ),
+    MethodFromCallable( "is_merged", [], [], __pullRequestIsMerged, SimpleTypePolicy( "bool" ) ),
+    MethodFromCallable( "merge", [], [ "commit_message" ], __mergePullRequest, SimpleTypePolicy( None ) ),
+)
+
+RepositoryKey = GithubObject(
+    "RepositoryKey",
+    BaseUrl( lambda obj: obj._repo._baseUrl() + "/keys/" + str( obj.id ) ),
+    InternalSimpleAttributes(
+        "url", "id", "title", "key",
+        "_repo", ### Ugly hack
+    ),
+    Editable( [ "title", "key" ], [] ),
+    Deletable()
 )
 
 Repository = GithubObject(
@@ -354,6 +483,26 @@ Repository = GithubObject(
 )
 Repository._addAttributePolicy( InternalObjectAttribute( "parent", Repository ) )
 Repository._addAttributePolicy( InternalObjectAttribute( "source", Repository ) )
+Repository._addAttributePolicy(
+    ExternalListOfObjects( "events", "event", Event,
+        ListGetable( [], [] )
+    ),
+)
+def __getNetworkEvents( repo ):
+    return [
+        Event( repo._github, attributes, lazy = True )
+        for attributes
+        in repo._github._dataRequest( "GET", "/networks/" + repo.owner.login + "/" + repo.name + "/events", None, None )
+    ]
+Repository._addAttributePolicy(
+    MethodFromCallable( "get_network_events", [], [], __getNetworkEvents, SimpleTypePolicy( "list of `Event`" ) )
+)
+Repository._addAttributePolicy(
+    ExternalListOfObjects( "issues/events", "issues_event", IssueEvent,
+        ListGetable( [], [], __modifyAttributesForObjectsReferingRepo ),
+        ElementGetable( [ "id" ], [], __modifyAttributesForObjectsReferingRepo ),
+    )
+)
 Repository._addAttributePolicy(
     ExternalListOfObjects( "forks", "fork", Repository,
         ListGetable( [], [] )
@@ -366,6 +515,16 @@ Repository._addAttributePolicy(
     SeveralAttributePolicies( [ ExternalSimpleAttribute( "languages", "dictionary of strings to integers" ) ], "Languages" )
 )
 Repository._addAttributePolicy( SeveralAttributePolicies( [
+    ExternalListOfObjects( "hooks", "hook", Hook,
+        ListGetable( [], [], __modifyAttributesForObjectsReferingRepo ),
+        ElementGetable( [ "id" ], [], __modifyAttributesForObjectsReferingRepo ),
+        ElementCreatable( [ "name", "config" ], [ "events", "active" ], __modifyAttributesForObjectsReferingRepo ),
+    ),
+    ExternalListOfObjects( "keys", "key", RepositoryKey,
+        ListGetable( [], [], __modifyAttributesForObjectsReferingRepo ),
+        ElementGetable( [ "id" ], [], __modifyAttributesForObjectsReferingRepo ),
+        ElementCreatable( [ "title", "key" ], [], __modifyAttributesForObjectsReferingRepo ),
+    ),
     ExternalListOfObjects( "collaborators", "collaborator", NamedUser,
         ListGetable( [], [] ),
         ElementAddable(),
@@ -385,7 +544,7 @@ Repository._addAttributePolicy( SeveralAttributePolicies( [
     ),
     ExternalListOfObjects( "git/commits", "git_commit", GitCommit,
         ElementGetable( [ "sha" ], [], __modifyAttributesForObjectsReferingRepo ),
-        ElementCreatable( [ "message", "tree", "parents" ], [ "author", "commiter" ], __modifyAttributesForObjectsReferingRepo )
+        ElementCreatable( [ "message", "tree", "parents" ], [ "author", "committer" ], __modifyAttributesForObjectsReferingRepo )
     ),
     ExternalListOfObjects( "git/trees", "git_tree", GitTree,
         ElementGetable( [ "sha" ], [], __modifyAttributesForObjectsReferingRepo ),
@@ -435,7 +594,7 @@ Repository._addAttributePolicy( SeveralAttributePolicies( [
     ),
     ExternalListOfObjects( "pulls", "pull", PullRequest,
         ListGetable( [], [ "state" ], __modifyAttributesForObjectsReferingRepo ),
-        ElementGetable( [ "id" ], [], __modifyAttributesForObjectsReferingRepo ),
+        ElementGetable( [ "number" ], [], __modifyAttributesForObjectsReferingRepo ),
         ElementCreatable( [ "title", "body", "base", "head" ], [], __modifyAttributesForObjectsReferingRepo ),
     ),
 ] ) )
@@ -480,11 +639,11 @@ NamedUser._addAttributePolicy(
 
 def __createForkForUser( user, repo ):
     assert isinstance( repo, Repository )
-    return Repository( user._github, user._github._dataRequest( "POST", repo._baseUrl + "/forks", None, None ), lazy = True )
+    return Repository( user._github, user._github._dataRequest( "POST", repo._baseUrl() + "/forks", None, None ), lazy = True )
 AuthenticatedUser._addAttributePolicy( SeveralAttributePolicies( [ MethodFromCallable( "create_fork", [ "repo" ], [], __createForkForUser, ObjectTypePolicy( Repository ) ) ], "Forking" ) )
 def __createForkForOrg( org, repo ):
     assert isinstance( repo, Repository )
-    return Repository( org._github, org._github._dataRequest( "POST", repo._baseUrl + "/forks", { "org": org.login }, None ), lazy = True )
+    return Repository( org._github, org._github._dataRequest( "POST", repo._baseUrl() + "/forks", { "org": org.login }, None ), lazy = True )
 Organization._addAttributePolicy( SeveralAttributePolicies( [ MethodFromCallable( "create_fork", [ "repo" ], [], __createForkForOrg, ObjectTypePolicy( Repository ) ) ], "Forking" ) )
 
 Team = GithubObject(
@@ -521,3 +680,82 @@ Repository._addAttributePolicy(
         ListGetable( [], [] )
     )
 )
+
+GistComment = GithubObject(
+    "GistComment",
+    BaseUrl( lambda obj: "/gists/comments/" + str( obj.id ) ),
+    InternalSimpleAttributes(
+        "id", "url", "body", "created_at",
+        "updated_at",
+    ),
+    InternalObjectAttribute( "user", NamedUser ),
+    Editable( [ "body" ], [] ),
+    Deletable(),
+)
+
+def __isStarred( gist ):
+    return gist._github._statusRequest( "GET", gist._baseUrl() + "/star", None, None ) == 204
+def __setStarred( gist ):
+    gist._github._statusRequest( "PUT", gist._baseUrl() + "/star", None, None )
+def __resetStarred( gist ):
+    gist._github._statusRequest( "DELETE", gist._baseUrl() + "/star", None, None )
+Gist = GithubObject(
+    "Gist",
+    BaseUrl( lambda obj: "/gists/" + str( obj.id ) ),
+    InternalSimpleAttributes(
+        "url", "id", "description", "public", "files", "comments", "html_url",
+        "git_pull_url", "git_push_url", "created_at", "forks", "history",
+        "updated_at",
+    ),
+    InternalObjectAttribute( "user", NamedUser ),
+    Editable( [], [ "description", "files" ] ),
+    Deletable(),
+    ExternalListOfObjects( "comments", "comment", GistComment,
+        ListGetable( [], [] ),
+        ElementGetable( [ "id" ], [] ),
+        ElementCreatable( [ "body" ], [] ),
+    ),
+    SeveralAttributePolicies( [
+        MethodFromCallable( "is_starred", [], [], __isStarred, SimpleTypePolicy( "bool" ) ),
+        MethodFromCallable( "set_starred", [], [], __setStarred, SimpleTypePolicy( None ) ),
+        MethodFromCallable( "reset_starred", [], [], __resetStarred, SimpleTypePolicy( None ) ),
+    ], "Starring" ),
+)
+def __createFork( gist ):
+    return Gist( gist._github, gist._github._dataRequest( "POST", gist._baseUrl() + "/fork", None, None ), lazy = True )
+Gist._addAttributePolicy(    SeveralAttributePolicies( [
+        MethodFromCallable( "create_fork", [], [], __createFork, ObjectTypePolicy( Gist ) ),
+    ], "Forking" ),
+)
+
+NamedUser._addAttributePolicy(
+    ExternalListOfObjects( "gists", "gist", Gist,
+        ListGetable( [], [] ),
+    )
+)
+
+AuthenticatedUser._addAttributePolicy(
+    ExternalListOfObjects( "gists", "gist", Gist,
+        ListGetable( [], [] ),
+        ElementCreatable( [ "public", "files", ], [ "description" ] ),
+        url = "/gists",
+    )
+)
+def __getStaredGists( user ):
+    return [
+        Gist( user._github, attributes, lazy = True )
+        for attributes in user._github._dataRequest( "GET", "/gists/starred", None, None )
+    ]
+AuthenticatedUser._addAttributePolicy(
+    MethodFromCallable( "get_starred_gists", [], [], __getStaredGists, SimpleTypePolicy( "list of `Gist`" ) ),
+)
+
+Event._addAttributePolicy(
+    InternalObjectAttribute( "repo", Repository ),
+)
+Event._addAttributePolicy(
+    InternalObjectAttribute( "actor", NamedUser ),
+)
+Event._addAttributePolicy(
+    InternalObjectAttribute( "org", Organization ),
+)
